<cfcomponent extends="base">
	
	<cffunction name="testScrubHTML">
		<cfset assertEquals("test123", getSecurityUtil().scrubHTML("test123"))>
	</cffunction>
	
	<cffunction name="testScrubHTMLSimple">
		<cfset var tags = {"p"={}, "i"={}, "em"={}, "strong"={}, "b"={}}>
		<cfset var result = "">
		<cfset assertEquals("test123", getSecurityUtil().scrubHTML("test123", tags))>
		<cfset result = getSecurityUtil().scrubHTML("<p>test123</p>", tags)>
		<cfset assert("<p>test123</p>" IS result, "Values do not match: #htmlEditFormat(result)#")>
	</cffunction>
	
	<cffunction name="testBR">
		<cfset var tags = {"br"={}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<br />", tags)>
		<cfset assert("<br />" IS result, "Values do not match: #htmlEditFormat(result)#")>
	</cffunction>
	
	<cffunction name="testAttrib">
		<cfset var tags = {"p"={"id"="match:[a-z0-9]*"}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<p id=""test"" />", tags)>
		<cfset assert("<p id=""test"" />" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testTagScrub">
		<cfset var tags = {"img"={"id"="match:[a-z0-9]*"}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("This is a test. <p id=""test"" />12345", tags)>
		<cfset assert("This is a test. 12345" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testMalformed">
		<cfset var tags = {"img"={"src"="match:^[a-z0-9]+$"}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("This is a test. <p id=""test"" /12345", tags)>
		<cfset assert("This is a test. " IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>

	<cffunction name="testMalformedTag">
		<cfset var tags = {"p"={}, "i"={}, "em"={}, "strong"={}, "b"={}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("Test <em /<body onload=alert(1)>", tags)>
		<cfset assert("Test <em>" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testGroup">
		<cfset var tags = {"p"={"class"="match:(bacon|pork|ham)"}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<p class=""chicken"">1<p class=""pork"">2", tags)>
		<cfset assert("<p>1<p class=""pork"">2" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testUri">
		<cfset var tags = {"img"={"src"="uri"}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<img src=""/index.cfm?id=1&x=2%20"" />", tags)>
		<cfset assert("<img src=""/index.cfm?id=1&x=2%20"" />" IS result, "Values do not match: #xmlformat(result)#")>
		<cfset result = getSecurityUtil().scrubHTML("<img src=""/index.cfm?id=1&x=2%20 f"" />", tags)>
		<cfset assert("<img />" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testUrl">
		<cfset var tags = {"img"={"src"="url"}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<img src=""https://ex-ample.com/index.cfm?id=1&x=2%20"" />", tags)>
		<cfset assert("<img src=""https://ex-ample.com/index.cfm?id=1&x=2%20"" />" IS result, "Values do not match: #xmlformat(result)#")>
		<cfset result = getSecurityUtil().scrubHTML("<img src=""javascript:foo"" />", tags)>
		<cfset assert("<img />" IS result, "Values do not match: #xmlformat(result)#")>
		
		<cfset result = getSecurityUtil().scrubHTML("<IMG SRC=""jav&##x09;ascript:alert('XSS');"">", tags)>
		<cfset assert("<IMG>" IS result, "Values do not match: #xmlformat(result)#")>
		<!---  --->
		<cfset result = getSecurityUtil().scrubHTML("<IMG SRC=&##0000106&##0000097&##0000118&##0000097&##0000115&##0000099&##0000114&##0000105&##0000112&##0000116&##0000058&##0000097&
##0000108&##0000101&##0000114&##0000116&##0000040&##0000039&##0000088&##0000083&##0000083&##0000039&##0000041>", tags)>
		<cfset assert("<IMG>" IS result, "Values do not match: #xmlformat(result)#")>
		<cfset result = getSecurityUtil().scrubHTML("<IMG SRC=""livescript:[code]"">", tags)>
		<cfset assert("<IMG>" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testRemove">
		<cfset var tags = {"p"={"id"="remove:[^a-zA-Z0-9]"}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<p id=""test@example.com"" />", tags)>
		<cfset assert("<p id=""testexamplecom"" />" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testDefaultPolicy">
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<table><thead><tr><th colspan=""2"">Test</th></tr></thead><tbody><tr><td><div /><hacker /></td></tr></tbody></table>")>
		<cfset assert("<table><thead><tr><th colspan=""2"">Test</th></tr></thead><tbody><tr><td><div /></td></tr></tbody></table>" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testGlobalPolicy">
		<cfset var tags = {"*"={"id"="remove:[^a-zA-Z0-9]"}, p={}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<p id=""test@example.com"" />", tags)>
		<cfset assert("<p id=""testexamplecom"" />" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testStyleColor">
		<cfset var tags = {p={style={color="css-color", "background-color"="css-color"}}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<p style=""color: red; background-color: ##f1f1f1; background-image:url(foo);"" />", tags)>
		<cfset assert("<p style=""color:red;background-color:##f1f1f1;"" />" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	<cffunction name="testStyleDimension">
		<cfset var tags = {p={style={margin="css-dimension", padding="css-dimension", "font-size"="css-dimension"}}}>
		<cfset var result = "">
		<cfset result = getSecurityUtil().scrubHTML("<p style=""margin:5px 6px; padding:10% 4% 10% 5%; font-size:12em; background-image:url(foo);"" />", tags)>
		<cfset assert("<p style=""margin:5px 6px;padding:10% 4% 10% 5%;font-size:12em;"" />" IS result, "Values do not match: #xmlformat(result)#")>
	</cffunction>
	
	
	
	
	
</cfcomponent>